@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013-2024
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;  Copyright (C) 2013-2025
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -39,11 +39,14 @@
   #:use-module (artanis websocket)
   #:use-module (artanis lpc)
   #:use-module (artanis security nss)
+  #:use-module (artanis i18n)
+  #:use-module (artanis cli)
   #:use-module (artanis version)
   #:use-module (web server)
   #:use-module (srfi srfi-1)
   #:use-module (ice-9 format)
-  #:re-export (;; env
+  #:use-module (ice-9 threads)
+  #:re-export ( ;; env
                http-status
 
                ;; page module
@@ -86,6 +89,7 @@
                :from-post
                :websocket
                :lpc
+               :i18n
 
                ;; db module
                DB-open
@@ -154,8 +158,6 @@
                scm->json-string
                json->scm
                json-string->scm
-               json-parser?
-               json-parser-port
 
                ;; upload
                mfd-simple-dump
@@ -218,6 +220,14 @@
   (when (and (not (linux-version>=? "3.9")) (get-conf '(server multi)))
     (error "It's required to have Linux-3.9+ to enable server.multi feature!")))
 
+(define (init-tmp)
+  (let ((tmp (format #f "/tmp/~a-~a" (current-appname)
+                     (get-random-from-dev #:length 4))))
+    (when (file-exists? tmp)
+      (cli-run* rm -fr ,tmp))
+    (mkdir tmp)
+    (current-tmp tmp)))
+
 ;; make sure to call init-server at the beginning
 (define* (init-server #:key (statics '(png jpg jpeg ico html js css))
                       (cache-statics? #f) (exclude '()))
@@ -225,53 +235,58 @@
   (init-hook)
   (init-config)
   (init-lpc)
+  (init-i18n)
+  (init-tmp)
   (check-invalid-config)
   (sigaction SIGPIPE SIG_IGN) ; surpass SIGPIPE signal since we want to handle EPIPE by self
   (sigaction SIGINT (lambda (i)
                       (run-when-sigint-hook)
                       (nss:pr-cleanup)
                       (format (artanis-current-output)
-                              "~%Fare you well, your server is cold.~%")
-                      (quit)))
-  (set! is-init-server-run? #t))
+                              "~%Fare you well, your server is cold.~%")))
+
+  ;; define default system error status
+  (http-status 408 (lambda () ""))
+
+  (is-init-server-run? #t))
 
 (define* (form-tag #:key (controller #f) (action #f) (method #f)
                    (class #f) (tag-class #f) (tag-id #f) (form-method "get"))
   (lambda tags
     (call-with-output-string
-      (lambda (port)
-        (format port "<form accept-charset='~a'" (get-conf '(server charset)))
-        (format port " action='~a'"
-                (call-with-output-string
-                  (lambda (port2)
-                    (display "/" port2)
-                    (and controller (format port2 "~a/" controller))
-                    (and action (format port2 "~a?" action))
-                    (and method (format port2 "method=~a" method))
-                    (and class (format port2 "&class=~a" class)))))
-        (format port " method='~a'" method)
-        (and tag-class (format port " class='~a'" tag-class))
-        (and tag-id (format port " id='~a'" tag-id))
-        (display ">\n" port)
-        (for-each (lambda (tag) (format port "~a~%" tag)) tags)
-        (format port "</form>~%")))))
+     (lambda (port)
+       (format port "<form accept-charset='~a'" (get-conf '(server charset)))
+       (format port " action='~a'"
+               (call-with-output-string
+                (lambda (port2)
+                  (display "/" port2)
+                  (and controller (format port2 "~a/" controller))
+                  (and action (format port2 "~a?" action))
+                  (and method (format port2 "method=~a" method))
+                  (and class (format port2 "&class=~a" class)))))
+       (format port " method='~a'" method)
+       (and tag-class (format port " class='~a'" tag-class))
+       (and tag-id (format port " id='~a'" tag-id))
+       (display ">\n" port)
+       (for-each (lambda (tag) (format port "~a~%" tag)) tags)
+       (format port "</form>~%")))))
 
 (define (make-general-tag tag)
   (lambda attrs
     (lambda (contents)
       (call-with-output-string
-        (lambda (port)
-          (format port "<~a" tag)
-          (let lp((next attrs))
-            (cond
-             ((null? next)
-              (display ">\n" port)
-              (display contents port)
-              (newline port)
-              (format port "</~a>" tag))
-             (else
-              (format port " ~a='~a'" (keyword->symbol (car next)) (cadr next))
-              (lp (cddr attrs))))))))))
+       (lambda (port)
+         (format port "<~a" tag)
+         (let lp((next attrs))
+           (cond
+            ((null? next)
+             (display ">\n" port)
+             (display contents port)
+             (newline port)
+             (format port "</~a>" tag))
+            (else
+             (format port " ~a='~a'" (keyword->symbol (car next)) (cadr next))
+             (lp (cddr attrs))))))))))
 
 (define label-tag (make-general-tag 'label))
 (define a-tag (make-general-tag 'a))
@@ -310,7 +325,7 @@
           (print-websocket-frame body)
           "<websocket-frame>"))
      (else (throw 'artanis-err 500 ->proper-body-display "Invalid body type `~a'!" body))))
-  (when (not is-init-server-run?)
+  (when (not (is-init-server-run?))
     (error "Sorry, but you have to run (init-server) in the begining of you main program!"))
   (and host (conf-set! '(host addr) host))
   (and port (conf-set! '(host port) port))
@@ -345,4 +360,14 @@
                        (format #t "[Request] ~a~%[Body] ~a~%" r (->proper-body-display b))
                        (server-handler r b))
                      server-handler)))
-    (establish-http-gateway handler)))
+    (let ((workers (get-conf '(server workers))))
+      (cond
+       ((= workers 1) (establish-http-gateway handler))
+       (else
+        ;; NOTE: Workers higher than 1 implies multi server mode.
+        (conf-set! '(server multi) #t)
+        ;; NOTE: We use process rather than thread to avoid the lock problems
+        (n-par-for-each workers
+                        (lambda (_)
+                          (establish-http-gateway handler))
+                        (iota workers)))))))
