@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2016,2017,2018,2019
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;  Copyright (C) 2014-2025
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -36,10 +36,13 @@
             *after-websocket-handshake-hook*
             *DB-conn-init-hook*
             *when-sigint-hook*
+            *refresh-hook*
             *sql-mapping-lookup-table*
             *artanis-entry*
             %current-toplevel
             current-toplevel
+            proper-toplevel
+            current-tmp
             find-ENTRY-path
             cmd:is-dry-run?
             cmd:is-force?
@@ -60,7 +63,6 @@
             %system-arch
             kernel-version>=?
             linux-version>=?
-            is-init-server-run?
             current-encoder
             *lpc-instance-pool*
             out-of-task-prompt?
@@ -71,7 +73,12 @@
             is-mmapped-file?
             register-mmapped-file!
             http-status
-            get-syspage-handler))
+            get-syspage-handler
+            register-cache-handler!
+            has-cache-handler?
+            is-init-server-run?
+            workers-pool
+            preparing-quit?))
 
 ;; WARNING:
 ;; For concurrency in green-thread, all these stuffs should be immutable
@@ -94,7 +101,7 @@
 ;; NOTE: The init queue size equals to (server wqlen).
 ;;       If all the available DB conn were blocked, a new DB conn will be
 ;;       created, and never closed but just recycled by *conn-pool*.
-(define *conn-pool* #f)
+(define *conn-pool* (make-parameter #f))
 
 (define *before-response-hook* (make-hook 2))
 (define *after-request-hook* (make-hook 2))
@@ -102,6 +109,7 @@
 (define *DB-conn-init-hook* (make-hook 1))
 (define *after-websocket-handshake-hook* (make-hook 2))
 (define *when-sigint-hook* (make-hook))
+(define *refresh-hook* (make-hook))
 
 ;; TODO: I don't have much time for it but it should be RB-Tree in the future
 (define *sql-mapping-lookup-table* (make-hash-table))
@@ -130,6 +138,11 @@
   (or (%current-toplevel)
       (find-ENTRY-path identity #t)))
 
+(define-syntax-rule (proper-toplevel)
+  (or (current-toplevel) "./"))
+
+(define current-tmp (make-parameter "Fatal: current-tmp was not initialized!"))
+
 ;; parameters for command
 (define cmd:is-dry-run? (make-parameter #f))
 (define cmd:is-force? (make-parameter #f))
@@ -150,9 +163,8 @@
 
 (define (current-appname) (and=> (current-toplevel) basename))
 
-(define *session-backend* #f)
-(define (change-session-backend! x) (set! *session-backend* x))
-(define (current-session-backend) *session-backend*)
+(define current-session-backend (make-parameter #f))
+(define (change-session-backend! x) (current-session-backend x))
 
 (define *proto-table* (make-hash-table))
 
@@ -215,8 +227,6 @@
 ;; either 'epoll or 'kqueue to use in cond-expand inside ragnarok
 ;; to select proper module for event-queue implementation.
 
-(define is-init-server-run? #f)
-
 (define current-encoder (make-parameter identity))
 
 (define *lpc-instance-pool* #f)
@@ -229,7 +239,7 @@
 (define (oh-define! lst) (set! *options-meta-handler-table* lst))
 (define (oh-set! k h)
   (set! *options-meta-handler-table*
-    (assq-set! *options-meta-handler-table* k h)))
+        (assq-set! *options-meta-handler-table* k h)))
 (define (oh-ref o)
   (assq-ref *options-meta-handler-table* o))
 
@@ -245,3 +255,16 @@
   (hashv-set! *sys-page-handlers* status thunk))
 (define (get-syspage-handler status)
   (hashv-ref *sys-page-handlers* status #f))
+
+(define *cache-handlers-table* (make-hash-table))
+(define (register-cache-handler! rule handler)
+  (hash-set! *cache-handlers-table* rule handler))
+(define (has-cache-handler? uid) (hash-ref *cache-handlers-table* uid))
+
+;; NOTE: We solve global access properly with parameters
+;; https://lists.gnu.org/archive/html/guile-user/2024-09/msg00043.html
+(define is-init-server-run? (make-parameter #f))
+
+(define workers-pool (make-parameter '()))
+
+(define preparing-quit? (make-parameter #f))
