@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014-2021,2024
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;  Copyright (C) 2014-2025
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -43,6 +43,7 @@
   #:use-module (artanis security nss)
   #:use-module (artanis irregex)
   #:use-module (artanis lpc)
+  #:use-module (artanis i18n)
   #:use-module (ice-9 match)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
@@ -59,6 +60,7 @@
              page-options
              page-delete
 
+             plugin-enable!
              :sql-mapping
              :str
              :conn
@@ -77,7 +79,8 @@
              :session
              :from-post
              :websocket
-             :lpc))
+             :lpc
+             :i18n))
 
 (define (http-options-add! method rule)
   (let* ((digest (string->sha-1 rule))
@@ -600,6 +603,27 @@
                        (lambda (h) (h rc 'spawn))))))))
     (else (throw 'artanis-err 500 auth-maker "Wrong pattern ~a" val))))
 
+(define (i18n-maker val rule keys)
+  (define-syntax-rule (from-url rc field)
+    (parameterize ((current-lang (params rc field)))
+      (make-i18n-handler)))
+  (define-syntax-rule (from-cookie rc key)
+    (let ((lang ((=> #:cookies rc 'value) key)))
+      (parameterize ((current-lang lang))
+        (make-i18n-handler))))
+  (define-syntax-rule (from-header rc)
+    (let ((lang (match (get-header rc 'accept-language)
+                  (((_ . lang) . rest) lang)
+                  (else ""))))
+      (parameterize ((current-lang (http-lang-tag->gnu-locale lang)))
+        (make-i18n-handler))))
+  (lambda (rc)
+    (match val
+      ((? string? field) (from-url rc field))
+      (`(cookie ,name) (from-cookie rc name))
+      ('header (from-header rc))
+      (else (throw 'artanis-err 500 i18n-maker "Invalid cmd `~a'!" val)))))
+
 ;; -------------------------------------------------------------------
 
 ;; NOTE: these short-cut-maker should never be exported!
@@ -612,7 +636,7 @@
 ;; 2. Arounded with ${..}, and there's "@" as its prefix, say, ${@xxx},
 ;;    it means "to get xxx from body" (POST way).
 (oh-define!
- `(;; a short way for sql-mapping from the bindings in rule
+ `( ;; a short way for sql-mapping from the bindings in rule
    ;; e.g #:sql-mapping "select * from table where id=${:id}"
    (#:sql-mapping . ,sql-mapping-maker)
 
@@ -767,11 +791,51 @@
    ;; 1. #:with-auth must be cooperated with #:session
    ;;    If you use #:with-auth without #:session, then it throws
    ;;    an exception.
-   (#:with-auth . ,with-auth-maker)))
+   (#:with-auth . ,with-auth-maker)
+
+   ;; Register i18n lang field
+   ;; NOTE: The lang must be full language name, like "en_US", not "en"
+   ;; Modes:
+   ;; 1. string, e.g:
+   ;;    (get "/index/:lang" #:i18n "lang" ...)
+   ;;    This means the lang field will be fetched from (params rc "lang")
+   ;;    NOTE: URL i18n is recommended for the sessionless services, like the
+   ;;          page view for common visitors.
+   ;; 2. `(cookie ,name)
+   ;;    (get "/index" #:i18n `(cookie ,name) ...)
+   ;;    This means the lang field will be fetched from cookie named `name'.
+   ;;    NOTE: The #:cookies mode will be automatically enabled.
+   ;;    NOTE: Cookies i18n is recommended for the implementation of services
+   ;;          with session configure like CMS dashboard. However, you need
+   ;;          to set a controller to set the i18n cookie.
+   ;; 3. HTTP header "Accept-Language"
+   ;;    NOTE: Only the first lang will be used, if client sends multiple.
+   ;;    (get "/index" #:i18n 'header ...)
+   ;;    NOTE: Header i18n is recommended for the services with sessionless
+   ;;          services like Web API, when you want to unify the URL without
+   ;;          adding extra lang field.
+   (#:i18n . ,i18n-maker)))
 
 (define-macro (meta-handler-register what)
-  `(define-syntax-rule (,(symbol-append ': what) rc args ...)
-     (=> ,(symbol->keyword what) rc args ...)))
+  `(define-syntax-rule (,(symbol-append ': what) ?rc args ...)
+     (=> ,(symbol->keyword what) ?rc args ...)))
+
+(define-syntax-rule (plugin-enable! name handler)
+  (begin
+    (oh-set! (symbol->keyword 'name) handler)
+    (module-define! (resolve-module '(conf plugins))
+                    (symbol-append ': 'name)
+                    (lambda (rc . args)
+                      (let* ((opt (symbol->keyword 'name))
+                             (oht (rc-oht rc))
+                             (h (and oht (hash-ref oht opt))))
+                        (cond
+                         (h (apply h rc args))
+                         (else
+                          (throw 'artanis-err 500 'plugin-enable!
+                                 "BUG: ~a wasn't initialized!" opt))))))
+    (module-export! (resolve-module '(conf plugins))
+                    (list (symbol-append ': 'name)))))
 
 ;; register all the meta handler
 (meta-handler-register sql-mapping)
@@ -786,6 +850,7 @@
 (meta-handler-register from-post)
 (meta-handler-register websocket)
 (meta-handler-register lpc)
+(meta-handler-register i18n)
 
 (define-syntax-rule (:cookies-set! rc ck k v)
   ((:cookies rc 'set) ck k v))
@@ -833,10 +898,11 @@
                 (when (not (hash-ref oht #:cookies))
                   (init-oht! oht #:cookies #t rule keys))
                 (init-oht! oht #:session #t rule keys))
-               ((eq? k #:session)
+               ((or (eq? k #:session) (eq? k #:i18n))
                 ;; #:session requires #:cookies
                 (when (not (hash-ref oht #:cookies))
                   (init-oht! oht #:cookies #t rule keys))))
+              ;; init the handler
               (hash-set! oht k (h v rule keys)))
              (else #f))))
         opts)
