@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
 ;;  Copyright (C) 2013,2014,2015,2017,2018,2019
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -23,9 +23,12 @@
   #:use-module (artanis irregex)
   #:use-module (artanis mime)
   #:use-module (artanis route)
+  #:use-module (artanis server scheduler)
+  #:use-module (artanis server server-context)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 iconv)
   #:use-module (ice-9 match)
+  #:use-module (ice-9 format)
   #:use-module (srfi srfi-1)
   #:use-module ((rnrs)
                 #:select (get-bytevector-all
@@ -39,14 +42,12 @@
   #:export (mfd-simple-dump
             make-mfd-dumper
             content-type-is-mfd?
-            parse-mfd-body
             mfd
             call-with-mfd-data
             find-mfd
             make-mfd
             mfd?
             is-mfds?
-            mfds-count
             mfd-dispos
             mfd-name
             mfd-filename
@@ -77,7 +78,8 @@
    type)) ; MIME type
 
 (define (mfd->utf8 rc mfd)
-  (subbv->string (rc-body rc) "utf-8" (mfd-begin mfd) (- (mfd-end mfd) 2)))
+  (subbv->string (rc-body rc) "utf-8"
+                 (mfd-begin mfd) (- (mfd-end mfd) (mfd-begin mfd) -1)))
 (define (find-mfd rc name mfds)
   (any (lambda (mfd)
          (cond
@@ -131,7 +133,7 @@
     ((k v)
      (-> `(,k ,@(string-split v #\;))))
     (else (throw 'artanis-err 400 ->mfd-header
-                 "->mfd-headers: Invalid MFD header!" line))))
+                 "->mfd-headers: Invalid MFD header ~a!" line))))
 
 ;; NOTE: convert boundary to bytevector before pass in.
 (define (parse-mfds boundary body)
@@ -152,23 +154,27 @@
         (cond
          ((blankline? start) (cons ret (1+ end))) ; end of headers
          (else
-          (let ((line (subbv->string body "utf-8" start end))
+          (let ((line (subbv->string body "utf-8" start (- end start)))
                 (llen (- end start -1)))
             (lp (+ start llen) (bv-read-line body (+ start llen))
                 (cons (->mfd-header line) ret)))))))
      (else (throw 'artanis-err 400 get-headers
-                  "Invalid Multi Form Data header!" body))))
+                  "Invalid Multi Form Data header: ~a!" body))))
   (define (get-content-end from)
     (define btable (build-bv-lookup-table boundary))
     (let lp((i from))
+      (when (zero? (modulo i (get-conf '(server bufsize))))
+        (oneshot-mention! (current-client))
+        (break-task))
       (cond
        ((and (< (+ i blen) len)
              (not (hash-ref btable (bytevector-u8-ref body (+ i blen -1)))))
         (lp (+ i blen)))
-       ((is-boundary? i) i)
+       ((is-boundary? i)
+        ;; NOTE: There'd be 3 chars need to be dropped, \r\n(head) and \r(tail)
+        (- i 3))
        (else (lp (1+ i))))))
   ;; ENHANCEMENT: use Fast String Matching to move forward quicker
-  ;;(lp (1+ (bv-read-line body i)))))))
   (let lp((i 0) (mfds '()))
     (cond
      ((is-end-line? i) mfds)
@@ -182,9 +188,9 @@
              (name (-> dispos "name"))
              (type (-> headers "Content-Type"))
              (mfd (make-mfd (car dispos) name filename start end type)))
-        (lp end (cons mfd mfds))))
+        (lp (+ end 3) (cons mfd mfds))))
      (else (throw 'artanis-err 422 parse-mfds
-                  "Wrong multipart form body!")))))
+                  "Wrong multipart form body ~a!" body)))))
 
 (define* (make-mfd-dumper body #:key (path (current-upload-path))
                           (uid #f) (gid #f) (path-mode #o775)
@@ -198,10 +204,10 @@
         (if (string? newname)
             newname
             (throw 'artanis-err 500 gen-new-name
-                   (format #f "Somehow your renamer generated a bad name ~a!"
-                           newname)))))
+                   "Somehow your renamer generated a bad name ~a!"
+                   newname))))
      (else (throw 'artanis-err 500 gen-new-name
-                  (format #f "Invalid renamer ~a!" renamer)))))
+                  "Invalid renamer ~a!" renamer))))
   (lambda* (mfd #:key (renamer #f) (repath #f))
     (let* ((oldname (mfd-filename mfd))
            (filename (or (gen-new-name oldname renamer) oldname))
@@ -229,9 +235,8 @@
 
 (define (mfd-size m)
   (and (mfd-filename m)
-       ;; NOTE: There'd be 3 chars need to be dropped, \r\n(head) and \r(tail)
-       ;; (- (- (mfd-end m) (mfd-begin m) -1) 3)
-       (- (mfd-end m) (mfd-begin m) 2)))
+       ;; end - start + 1
+       (- (mfd-end m) (mfd-begin m) -1)))
 
 ;; NOTE: we won't limit file size here, since it should be done in server reader
 (define* (store-uploaded-files rc #:key (path (current-upload-path))
@@ -239,22 +244,22 @@
                                (mode #o664) (path-mode #o775) (sync #f))
   (define (get-slist mfds) (filter-map mfd-size mfds))
   (define (get-flist mfds) (filter-map mfd-filename mfds))
+  (define (finalize-mfds boundry)
+    (let ((mfds (parse-mfds (string->utf8 (string-append "--" boundry))
+                            (rc-body rc)))
+          (dumper (make-mfd-dumper (rc-body rc) #:path path #:mode mode
+                                   #:uid uid #:gid gid #:path-mode path-mode
+                                   #:sync sync)))
+      (catch #t
+        (lambda () (for-each dumper mfds))
+        (lambda e (format (current-error-port) "[WARNING] Failed to dump mfds! ~a~%" e)))
+      (if simple-ret?
+          (if need-mfd? mfds 'success)
+          (if need-mfd?
+              `(success ,mfds ,(get-slist mfds) ,(get-flist mfds))
+              `(success ,(get-slist mfds) ,(get-flist mfds))))))
   (cond
-   ((content-type-is-mfd? rc)
-    => (lambda (boundry)
-         (let ((mfds (parse-mfds (string->utf8 (string-append "--" boundry))
-                                 (rc-body rc)))
-               (dumper (make-mfd-dumper (rc-body rc) #:path path #:mode mode
-                                        #:uid uid #:gid gid #:path-mode path-mode
-                                        #:sync sync)))
-           (catch #t
-             (lambda () (for-each dumper mfds))
-             (lambda e (format (current-error-port) "[WARNING] Failed to dump mfds! ~a~%" e)))
-           (if simple-ret?
-               (if need-mfd? mfds 'success)
-               (if need-mfd?
-                   `(success ,mfds ,(get-slist mfds) ,(get-flist mfds))
-                   `(success ,(get-slist mfds) ,(get-flist mfds)))))))
+   ((content-type-is-mfd? rc) => finalize-mfds)
    (else 'none)))
 
 (define-record-type mfds-operator
@@ -262,16 +267,22 @@
    mfds ; the list of mfd
    dumper)) ; the dumper
 
-(define (mfds-op-ref rc mo key)
+(define* (mfds-op-ref rc mo key #:key (bv? #f))
   (let* ((mfds (mfds-operator-mfds mo))
          (value-mfd (any (lambda (mfd)
                            (and (string=? key (mfd-name mfd))
                                 mfd))
                          mfds)))
     (if value-mfd
-        (mfd->utf8 rc value-mfd)
+        (if bv?
+            (bv-slice
+             (rc-body rc)
+             (mfd-begin value-mfd)
+             :
+             (mfd-end value-mfd))
+            (mfd->utf8 rc value-mfd))
         (throw 'artanis-err 500 mfds-op-ref
-               (format #f "No mfd field named ~a!" key)))))
+               "No mfd field named ~a!" key))))
 
 (define (mfds-op-store mo . args)
   (let ((file-mfds (mfds-operator-mfds mo))
@@ -322,13 +333,13 @@
   (define boundary (format #f "----------Artanis-~a" (get-random-from-dev #:uppercase #t)))
   (define-syntax-rule (->dispos name filename mime)
     (call-with-output-string
-      (lambda (port)
-        (format port "--~a\r\n" boundary)
-        (format port "Content-Disposition: form-data; name=~s" name)
-        (and filename
-             (format port "; filename=~s" filename))
-        (and mime (format port "\r\nContent-Type: ~a" mime))
-        (display "\r\n\r\n" port))))
+     (lambda (port)
+       (format port "--~a\r\n" boundary)
+       (format port "Content-Disposition: form-data; name=~s" name)
+       (and filename
+            (format port "; filename=~s" filename))
+       (and mime (format port "\r\nContent-Type: ~a" mime))
+       (display "\r\n\r\n" port))))
   (define-syntax-rule (->file file)
     (if (file-exists? file)
         (call-with-input-file file get-bytevector-all)
@@ -340,15 +351,22 @@
           (datas (assoc-ref pattern 'data)))
       (append (-> files build-file-mfd)
               (-> datas build-data-mfd))))
+  (define (val-len val)
+    (bytevector-length
+     (cond
+      ((string? val) (string->bytevector val "iso8859-1"))
+      ((bytevector? val) val)
+      (else (throw 'artanis-err 500 build-data-mfd "val-len: Invalid value ~a" val)))))
   (define (build-data-mfd p)
     (match p
       ((name value)
-       (make-mfd (->dispos name #f #f)
-                 name
-                 #f
-                 0
-                 0
-                 #f))
+       (cons (make-mfd (->dispos name #f #f)
+                       name
+                       #f
+                       0
+                       (1- (val-len value))
+                       #f)
+             value))
       (else (throw 'artanis-err 500 build-data-mfd "Invalid pattern: ~a~%" p))))
   (define (build-file-mfd p)
     (match p
